from cmdcheatsheet.command import group_commands_by_name
from cmdcheatsheet.json_store import get_commands, get_commands_by_includes_command_name
from cmdcheatsheet.logger import error, command_details


def display_commands(display_index=False):
    command_dict = group_commands_by_name()
    id_column_size = calc_id_column_size(get_commands())
    for value in command_dict.values():
        for item in value:
            command_details(item, display_index, id_column_size)

def display_command_by_name(command_name, display_index=False):
    commands_to_display = get_commands_by_includes_command_name(command_name)
    if commands_to_display:
        id_column_size = calc_id_column_size(commands_to_display)
        for command in commands_to_display:
           command_details(command, display_index, id_column_size)
    else:
        error("Command not found.")

def calc_id_column_size(commands):
    command_ids = [c.id for c in commands]
    highest_command_id = max(command_ids) if command_ids else ""
    return len(str(highest_command_id))



from rich import print
from rich.console import Console
from rich.table import Table
from cmdcheatsheet.json_store import get_commands, get_commands_by_includes_command_name
from cmdcheatsheet.logger import red, blue, green, yellow
from cmdcheatsheet.command import get_command_name_list


def display_table_view(command=''):
    all_commands = get_commands_by_includes_command_name(command) if command else get_commands()
    console = Console()

    table = Table(show_header=True, header_style=red)

    table.add_column("Id")
    table.add_column("Command")
    table.add_column("Description")

    for c in all_commands:
        table.add_row(
            f'[{yellow}]{str(c.id)}',
            f'[{blue}]{c.command}',
            f'[{green}]{c.description}'
        )

    console.print(table)


def display_command_name_list(column_amount):
    commands = get_command_name_list()
    chunked_list = []

    chunk_size = column_amount if len(commands) >= column_amount else len(commands)

    if not chunk_size:
        print(f'[{red}] No commands have been found yet')
        return

    for i in range(0, len(commands), chunk_size):
        chunked_list.append(commands[i:i+chunk_size])

    console = Console()

    table = Table(show_header=False, header_style=red)

    for i in range(chunk_size):
        table.add_column()

    for chunk in chunked_list:
        row = []
        for cn in range(0, chunk_size):
            row.append(f'[{green}]{chunk[cn] if len(chunk) > cn else ""}')
        table.add_row(*row) 

    console.print(table)



from cmdcheatsheet.logger import help_details


def help(command_list):
    for c in command_list:
        commands_to_help = []
        
        for responsible_command in c.responsible_commands:
            command_arguments = [f'<{a.name}{":optional" if not a.required else ""}>' for a in c.command_arguments]
            arguments = (' '  + ' '.join(command_arguments)) if command_arguments else ''
            commands_to_help.append(f'cmd {responsible_command}{arguments}')

        help_details(', '.join(commands_to_help), c.description)




import os
import json
from dataclasses import asdict
from cmdcheatsheet.models import Command
from cmdcheatsheet.logger import error

user_home_dir = os.path.expanduser('~')
json_filename = "commands.json"
json_file_dir = f"{user_home_dir}/.config/cmdcheatsheet/{json_filename}"


def setup_config():
    try:
        if not os.path.exists(json_file_dir):
            os.makedirs(os.path.dirname(json_file_dir), exist_ok=True)
            with open(json_file_dir, 'w') as file_object:
                json.dump([], file_object, indent=4)
    except Exception as _:
        error("Error with config creation")

def get_commands():
    setup_config()
    with open(json_file_dir) as f:
      commands = json.load(f)
    return [Command(c.get('command'), c.get('description'), c.get('id')) for c in commands]

def get_index():
    commands = get_commands()
    if not commands:
        return 1
    else:
        last_command = commands[-1]
        return last_command.id + 1

def get_commands_by_includes_command_name(command_name):
    commands = []
    for command in get_commands():
        command_split = command.command.replace(',', '').split(' ')
        if command_name in command_split:
            commands.append(command)
    return commands

def save_all_commands(commands):
    commands_to_save = [asdict(c) for c in commands]
    with open(json_file_dir, 'w') as file_object:
        json.dump(commands_to_save, file_object, indent=4)



from cmdcheatsheet.json_store import get_commands, save_all_commands, get_index
from cmdcheatsheet.logger import info, error


# TODO: this and `group_commands_by_name` might be both refactored
# as they use too much the same code
def get_command_name_list():
    commands = get_commands()
    command_names = []

    for command in commands:
        command_split = command.command.split(' ')
        command_name = command_split[1] if command_split[0] == 'sudo' else command_split[0]

        if command_name.endswith(','):
            command_name = command_name.replace(',', '')

        if command_name not in command_names:
            command_names.append(command_name)

    return command_names


def group_commands_by_name():
    commands = get_commands()
    command_dict = {}

    for command in commands:
        command_split = command.command.split(' ')
        command_name = command_split[1] if command_split[0] == 'sudo' else command_split[0]

        if command_name.endswith(','):
            command_name = command_name.replace(',', '')

        if command_dict.get(command_name) is None:
            command_dict[command_name] = [command]
        else:
            command_dict.get(command_name).append(command)

    return command_dict


def add_command(command): 
    commands = get_commands()
    command_to_add = find_command_by_name(commands, command.command)

    if command_to_add is None:
        command.id = get_index()
        commands.append(command)
        save_all_commands(commands)
        info(f"Command \"{command.command}\" added.")
        return

    info(f"Command \"{command.command}\" already exists.")


def delete_command(command_id):
    commands = get_commands()
    command_to_delete = find_command_by_id(commands, command_id)

    if command_to_delete is not None:
        commands.remove(command_to_delete)
        save_all_commands(commands)
        info(f"Command with id: {command_id} removed.")
        return

    error(f"Command with id: {command_id} not found.")


def find_command_by_name(commands, command_value):
    return next((c for c in commands if c.command == command_value), None)


def find_command_by_id(commands, id):
    return next((c for c in commands if c.id == id), None)


def update_command(command):
    commands = get_commands()
    command_to_update = find_command_by_id(commands, command.id)

    if command_to_update is not None:
        command_to_update.command = command.command
        command_to_update.description = command.description
        save_all_commands(commands)
        info(f"Command with id: {command.id} updated.")
        return

    error(f"Command with id: {command.id} not found.")



#!/bin/python3
import sys

from cmdcheatsheet.logger import error
from cmdcheatsheet.command_list import get_command_by_name


def main():
    if len(sys.argv) == 1:
        sys.argv.append('-p')

    command = sys.argv[1]
    args = sys.argv[2:]


    try:
        command_to_invoke = get_command_by_name(command)
        if command_to_invoke:
            command_to_invoke.handle(args)
        else:
            error("Please provide the valid command name.")

    except Exception as e:
        error(e)

if __name__ == '__main__':
    main()



from cmdcheatsheet.models import CommandDetails, Command, CommandArgument
from cmdcheatsheet.display import *
from cmdcheatsheet.display_table import *
import cmdcheatsheet.command as command_actions
from cmdcheatsheet.help import help


class SimpleList(CommandDetails):
    def __init__(self):
       super().__init__(['-p', 'ls'], 'Display all commands.')

    def handler(self, _):
        display_commands()


class SimpleDetailedList(CommandDetails):
    def __init__(self):
       super().__init__(['-i', 'lsi'], 'Display all commands, including all details such as ids, etc., all commands.')

    def handler(self, _):
        display_commands(True)
        

class TableView(CommandDetails):
    def __init__(self):
       super().__init__(['-t', 'lst'], 'Display all commands using a table view.')

    def handler(self, _):
        display_table_view()


class AddCommand(CommandDetails):
    def __init__(self):
       super().__init__(
            ['-a', '-c'],
            'Add new command to the list.',
            [CommandArgument('command'), CommandArgument('description')])

    def handler(self, args):
        command = Command(command=args[0], description=args[1])
        command_actions.add_command(command)

    
class UpdateCommand(CommandDetails):
    def __init__(self):
       super().__init__(
            ['-u', '-e'],
            'Update a <command> by id.',
            [CommandArgument('id'), CommandArgument('name'), CommandArgument('description')])

    def handler(self, args):
        command = Command(id=int(args[0]), command=args[1], description=args[2])
        command_actions.update_command(command)


class DeleteCommand(CommandDetails):
    def __init__(self):
       super().__init__(
            ['-d', '-r'],
            'Delete a <command> by id.',
            [CommandArgument('id')])

    def handler(self, args):
        command_actions.delete_command(int(args[0]))


class SearchCommand(CommandDetails):
    def __init__(self):
       super().__init__(
            ['-s', '-f'],
            'Search for a command.',
            [CommandArgument('command')])

    def handler(self, args):
        display_command_by_name(args[0])


class SearchCommandDetailed(CommandDetails):
    def __init__(self):
       super().__init__(
            ['-si', '-fi'],
            'Search for a command and include all details, such as ids, etc.',
            [CommandArgument('command')])

    def handler(self, args):
        display_command_by_name(args[0], True)


class SearchCommandTableView(CommandDetails):
    def __init__(self):
       super().__init__(
            ['-st', '-ft'],
            'Search for a command and show it using a table view.',
            [CommandArgument('command')])

    def handler(self, args):
        display_table_view(command=args[0])


class NameListCommand(CommandDetails):
    def __init__(self):
       super().__init__(
            ['-nl', 'lsnl'],
            'Show all stored command names.',
            [CommandArgument('number_of_columns', False)])

    def handler(self, args):
        display_command_name_list(int(args[0]) if len(args) > 0 else 5)


class HelpCommand(CommandDetails):
    def __init__(self):
       super().__init__(['-h', 'help', '--help'], 'Show a program help notes.')

    def handler(self, _):
        help(command_list)

command_list = [
    SimpleList(),
    SimpleDetailedList(),
    TableView(),
    AddCommand(),
    UpdateCommand(),
    DeleteCommand(),
    SearchCommand(),
    SearchCommandDetailed(),
    SearchCommandTableView(),
    NameListCommand(),
    HelpCommand()
]

def get_command_by_name(command_name):
    return next((c for c in command_list if c.need_to_handle(command_name)), None)

